// GaugeView.cpp : implementation of the CGaugeView class
//

#include "stdafx.h"
#include "Gauge.h"

#include "GaugeDoc.h"
#include "GaugeView.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CGaugeView

IMPLEMENT_DYNCREATE(CGaugeView, CView)

BEGIN_MESSAGE_MAP(CGaugeView, CView)
	//{{AFX_MSG_MAP(CGaugeView)
	ON_WM_TIMER()
	ON_WM_ERASEBKGND()
	ON_COMMAND(ID_VIEW_FONT, OnViewFont)
	ON_COMMAND(ID_VIEW_INCHES, OnViewInches)
	ON_COMMAND(ID_VIEW_MM, OnViewMm)
	ON_COMMAND(ID_VIEW_BOTH, OnViewBoth)
	ON_COMMAND(ID_VIEW_GAUGEZERO, OnViewGaugezero)
	ON_COMMAND(ID_VIEW_LOCALZERO, OnViewLocalzero)
	ON_COMMAND(ID_VIEW_BOTHZERO, OnViewBothzero)
	ON_UPDATE_COMMAND_UI(ID_VIEW_INCHES, OnUpdateViewInches)
	ON_UPDATE_COMMAND_UI(ID_VIEW_MM, OnUpdateViewMm)
	ON_UPDATE_COMMAND_UI(ID_VIEW_BOTH, OnUpdateViewBoth)
	ON_UPDATE_COMMAND_UI(ID_VIEW_GAUGEZERO, OnUpdateViewGaugezero)
	ON_UPDATE_COMMAND_UI(ID_VIEW_LOCALZERO, OnUpdateViewLocalzero)
	ON_UPDATE_COMMAND_UI(ID_VIEW_BOTHZERO, OnUpdateViewBothzero)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CGaugeView construction/destruction

CGaugeView::CGaugeView()
{
}

CGaugeView::~CGaugeView()
{
}

BOOL CGaugeView::PreCreateWindow(CREATESTRUCT& cs)
{
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs

	return CView::PreCreateWindow(cs);
}

/////////////////////////////////////////////////////////////////////////////
// CGaugeView drawing

void CGaugeView::OnDraw(CDC* pDC)
{
	CGaugeDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);


	CFont *fontOld=pDC->SelectObject(&m_font);

	CString s;
	pDC->SetTextAlign(TA_RIGHT);
	int x=0;
	int y=0;
	CRect r;
	r.SetRect(x,y,x+m_iCellWidth,y+m_iCellHeight);
	if(m_eViewZeros==eGauge || m_eViewZeros==eBothZeros) {
		if(m_eViewUnits==eInches || m_eViewUnits==eBoth) {
			s=pDoc->m_sIN+" in";
			pDC->ExtTextOut(r.right,r.top,ETO_CLIPPED|ETO_OPAQUE,r,s,s.GetLength(),NULL);
			y=r.bottom;
		}
		if(m_eViewUnits==eMM || m_eViewUnits==eBoth) {
			r.SetRect(x,y,x+m_iCellWidth,y+m_iCellHeight);
			s=pDoc->m_sMM+" mm";
			pDC->ExtTextOut(r.right,r.top,ETO_CLIPPED|ETO_OPAQUE,r,s,s.GetLength(),NULL);
		}
		x=r.right; y=0;
	}
	if(m_eViewZeros==eLocal || m_eViewZeros==eBothZeros) {
		if(m_eViewUnits==eInches || m_eViewUnits==eBoth) {
			r.SetRect(x,y,x+m_iCellWidth,y+m_iCellHeight);
			s=pDoc->m_sIN2+" in";
			pDC->ExtTextOut(r.right,r.top,ETO_CLIPPED|ETO_OPAQUE,r,s,s.GetLength(),NULL);
			y=r.bottom;
		}
		if(m_eViewUnits==eMM || m_eViewUnits==eBoth) {
			r.SetRect(x,y,x+m_iCellWidth,y+m_iCellHeight);
			s=pDoc->m_sMM2+" mm";
			pDC->ExtTextOut(r.right,r.top,ETO_CLIPPED|ETO_OPAQUE,r,s,s.GetLength(),NULL);
		}
	}

	pDC->SelectObject(fontOld);
}

/////////////////////////////////////////////////////////////////////////////
// CGaugeView diagnostics

#ifdef _DEBUG
void CGaugeView::AssertValid() const
{
	CView::AssertValid();
}

void CGaugeView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}

CGaugeDoc* CGaugeView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CGaugeDoc)));
	return (CGaugeDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CGaugeView message handlers

void CGaugeView::OnInitialUpdate() 
{
	CView::OnInitialUpdate();
	
	SetTimer(1,100,NULL);
	
	m_eViewUnits=eBoth;	
	m_eViewZeros=eGauge;

	m_font.CreateFont(50,0,0,0,FW_MEDIUM, 0,0,0, ANSI_CHARSET, OUT_DEFAULT_PRECIS,
		CLIP_DEFAULT_PRECIS, DEFAULT_QUALITY, DEFAULT_PITCH|FF_DONTCARE,
		"Courier New"); //"OCR A Extended"); //"SF Digital Readout Medium");	//"Comic Sans MS" );

	SizeToFont();
}

void CGaugeView::OnTimer(UINT nIDEvent) 
{
	GetDocument()->ReadCom();
	
	CView::OnTimer(nIDEvent);
}

void CGaugeView::OnUpdate(CView* pSender, LPARAM lHint, CObject* pHint) 
{
	switch(lHint) {
		case 1:	// Inches / Gauge
			if((m_eViewUnits==eInches || m_eViewUnits==eBoth) &&
				(m_eViewZeros==eGauge || m_eViewZeros==eBothZeros)) {
				CRect r;
				r.left=0;
				r.top=0;
				r.right=r.left+m_iCellWidth;
				r.bottom=r.top+m_iCellHeight;
				InvalidateRect((LPCRECT)r);
			}
			break;
		case 2:	// MM / Gauge
			if((m_eViewUnits==eMM || m_eViewUnits==eBoth) &&
				(m_eViewZeros==eGauge || m_eViewZeros==eBothZeros)) {
				CRect r;
				r.left=0;
				r.top=m_eViewUnits==eBoth?m_iCellHeight:0;
				r.right=r.left+m_iCellWidth;
				r.bottom=r.top+m_iCellHeight;
				InvalidateRect((LPCRECT)r);
			}
			break;
			break;
		case 3:	// Inches / Local
			if((m_eViewUnits==eInches || m_eViewUnits==eBoth) &&
				(m_eViewZeros==eLocal || m_eViewZeros==eBothZeros)) {
				CRect r;
				r.left=m_eViewZeros==eBothZeros?m_iCellWidth:0;
				r.top=0;
				r.right=r.left+m_iCellWidth;
				r.bottom=r.top+m_iCellHeight;
				InvalidateRect((LPCRECT)r);
			}
			break;
		case 4:	// MM / Local
			if((m_eViewUnits==eMM || m_eViewUnits==eBoth) &&
				(m_eViewZeros==eLocal || m_eViewZeros==eBothZeros)) {
				CRect r;
				r.left=m_eViewZeros==eBothZeros?m_iCellWidth:0;
				r.top=m_eViewUnits==eBoth?m_iCellHeight:0;
				r.right=r.left+m_iCellWidth;
				r.bottom=r.top+m_iCellHeight;
				InvalidateRect((LPCRECT)r);
			}
			break;
		default:
			Invalidate();
			break;
	}
}

BOOL CGaugeView::OnEraseBkgnd(CDC* pDC) 
{
	return 0;

	//return CView::OnEraseBkgnd(pDC);
}

void CGaugeView::OnViewFont() 
{
	CFontDialog dlg(NULL,CF_INITTOLOGFONTSTRUCT|CF_SCREENFONTS|CF_FIXEDPITCHONLY);
	m_font.GetLogFont(dlg.m_cf.lpLogFont);

	dlg.DoModal();

	m_font.DeleteObject();
	m_font.CreateFontIndirect(dlg.m_cf.lpLogFont);

	SizeToFont();

	Invalidate();
}

void CGaugeView::SizeToFont(void)
{
	CDC* dc=GetDC();
	CFont* fontOld=dc->SelectObject(&m_font);
	TEXTMETRIC tm;
	dc->GetTextMetrics(&tm);
	dc->SelectObject(fontOld);
	ReleaseDC(dc);
	TRACE2("Width: %i  Height: %i\n",tm.tmAveCharWidth,tm.tmHeight);
	m_iCellHeight=tm.tmHeight;
	m_iCellWidth=tm.tmAveCharWidth*12;

	CRect rectMain,rectView;
	AfxGetMainWnd()->GetWindowRect(&rectMain);
	GetClientRect(&rectView);
	const int iDeltaX=rectMain.Width()-rectView.Width();
	const int iDeltaY=rectMain.Height()-rectView.Height();
	TRACE2("DeltaX: %i  DeltaY: %i\n",iDeltaX,iDeltaY);
	rectMain.right=rectMain.left+iDeltaX+m_iCellWidth*(m_eViewZeros==eBothZeros?2:1);
	rectMain.bottom=rectMain.top+iDeltaY+m_iCellHeight*(m_eViewUnits==eBoth?2:1);
	AfxGetMainWnd()->MoveWindow(rectMain,TRUE);
}

void CGaugeView::OnViewInches() 
{
	m_eViewUnits=eInches;
	SizeToFont();
	Invalidate();
}

void CGaugeView::OnUpdateViewInches(CCmdUI* pCmdUI) 
{
	pCmdUI->SetRadio(m_eViewUnits==eInches);
}

void CGaugeView::OnViewMm() 
{
	m_eViewUnits=eMM;
	SizeToFont();
	Invalidate();
}

void CGaugeView::OnUpdateViewMm(CCmdUI* pCmdUI) 
{
	pCmdUI->SetRadio(m_eViewUnits==eMM);
}

void CGaugeView::OnViewBoth() 
{
	m_eViewUnits=eBoth;	
	SizeToFont();
	Invalidate();
}

void CGaugeView::OnUpdateViewBoth(CCmdUI* pCmdUI) 
{
	pCmdUI->SetRadio(m_eViewUnits==eBoth);
}

void CGaugeView::OnViewGaugezero() 
{
	m_eViewZeros=eGauge;
	SizeToFont();
	Invalidate();
}

void CGaugeView::OnUpdateViewGaugezero(CCmdUI* pCmdUI) 
{
	pCmdUI->SetRadio(m_eViewZeros==eGauge);
}

void CGaugeView::OnViewLocalzero() 
{
	m_eViewZeros=eLocal;
	SizeToFont();
	Invalidate();
}

void CGaugeView::OnUpdateViewLocalzero(CCmdUI* pCmdUI) 
{
	pCmdUI->SetRadio(m_eViewZeros==eLocal);
}

void CGaugeView::OnViewBothzero() 
{
	m_eViewZeros=eBothZeros;
	SizeToFont();
	Invalidate();
}

void CGaugeView::OnUpdateViewBothzero(CCmdUI* pCmdUI) 
{
	pCmdUI->SetRadio(m_eViewZeros==eBothZeros);
}
